;**********************************************
; Port config:
; * PortC  : Out - display-data
;
; * RB4    : SMBus - SDA
; * RB5    : SMBus - SCL
; * RB6    : User-input
;**********************************************

	errorlevel  -302


	#include "config.inc" 
  
	__CONFIG       _CP_OFF & _CPD_OFF & _WDT_OFF & _BOR_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT  & _MCLRE_OFF & _FCMEN_OFF & _IESO_OFF
	
	; from the math module
	extern	REG_X
	extern	REG_Y
	extern	REG_Z
	extern	M_DIV
	extern	M_MUL
	extern	M_SUB
	extern	M_ADD
	extern	M_CMP
	
	; from rs232.asm
	extern	RS232_Init
	extern	RS232_Send_W
	
	; from the display module
	extern	Display_init
	extern	Display_clear
	extern	Display_set_pos_line1
	extern	Display_set_pos_line2
	extern	Display_digit_char
	extern	Display_write_char
	
	; from the smbus module
	extern	SMBus_Init
	extern	MLX90614_ReadSensorTemp
	extern	MLX90614_ReadObjectTemp
	extern	TempSensorL
	extern	TempSensorH
	extern	TempObjectL
	extern	TempObjectH
	
	udata
DelayCounter1	res 1
DelayCounter2	res 1
DelayCounter3	res 1
Temp			res 4 ; Temp[0] & Temp[1] => value before comma; Temp[2] => value behind comma; Temp[3],0 neg/pos flag

Reset	CODE	0x0
	pagesel	_init
	goto	_init


	code

_init
	; set the clockspeed
	banksel	OSCCON
	if CLOCKSPEED == .8000000
		movlw	b'01110000'
	else
		if CLOCKSPEED == .4000000
			movlw	b'01100000'
		else
			error	"Unsupported clockspeed"
		endif
	endif
	movwf	OSCCON
	
	; setup option register
	banksel	OPTION_REG
	movlw	b'00000000'	
		;	  ||||||||---- PS0 - Timer 0: 
		;	  |||||||----- PS1
		;	  ||||||------ PS2
		;	  |||||------- PSA -  Assign prescaler to Timer0
		;	  ||||-------- TOSE - LtoH edge
		;	  |||--------- TOCS - Timer0 uses IntClk
		;	  ||---------- INTEDG - falling edge RB0
		;	  |----------- NOT_RABPU - pull-ups enabled
	movwf	OPTION_REG
	
	; all ports to digital
	banksel	ANSEL
	clrf	ANSEL
	clrf	ANSELH
	
	; set entire PORTA to output
	BANKSEL TRISA
	clrf	TRISA

	; set entire PORTB to output
	BANKSEL TRISB
	clrf	TRISB

	; set entire PORTC to output
	BANKSEL TRISC
	clrf	TRISC
	
	; set all output ports to 0
	banksel	PORTA
	clrf	PORTA
	clrf	PORTB
	clrf	PORTC
	
	; init display module
	call	Display_init
	
	; init rs232 module
	call	RS232_Init

	; init smbus module
	call	SMBus_Init
	
_main
	call	Delay_1s

	call	Display_clear
	
	; read the current temps
	call	MLX90614_ReadSensorTemp
	call	MLX90614_ReadObjectTemp
	
	; send the values over RS232
	;movfw	TempSensorH
	;call	RS232_Send_W
	;movfw	TempSensorL
	;call	RS232_Send_W
	; write the sensor temp to display
	movfw	TempSensorH
	movwf	Temp+1
	movfw	TempSensorL
	movwf	Temp
	call	Convert_to_c_degrees	; convert values
	call	Send_temp_to_display
	
	; send the values over RS232
	;movfw	TempObjectH
	;call	RS232_Send_W
	;movfw	TempObjectL
	;call	RS232_Send_W
	; write the objecy temp to display
	movfw	TempObjectH
	movwf	Temp+1
	movfw	TempObjectL
	movwf	Temp
	call	Convert_to_c_degrees	; convert values
	movlw	.0
	call	Display_set_pos_line2
	call	Send_temp_to_display
	
	goto _main

; produces always 7 chars ("-XXX.XX") in the display
Send_temp_to_display
	; check if the value is negative or not
	btfss	Temp+3, 0
	goto	Send_temp_to_display_pos
	goto	Send_temp_to_display_neg
Send_temp_to_display_neg
	movlw	'-'
	call	Display_write_char
	goto	Send_temp_to_display_value
Send_temp_to_display_pos
	movlw	'+'
	call	Display_write_char
	goto	Send_temp_to_display_value

	; next, write the value
Send_temp_to_display_value
	movfw	Temp
	movwf	REG_Z
	movfw	Temp+1
	movwf	REG_Z+1

	; send the three decial digits before the comma
	movlw	HIGH 	.100
	movwf	REG_X+1
	movlw	LOW 	.100
	movwf	REG_X
	call	M_DIV
	movfw	REG_Y
	call	Display_digit_char

	movlw	HIGH 	.10
	movwf	REG_X+1
	movlw	LOW 	.10
	movwf	REG_X
	call	M_DIV
	movfw	REG_Y
	call	Display_digit_char

	movlw	HIGH 	.1
	movwf	REG_X+1
	movlw	LOW 	.1
	movwf	REG_X
	call	M_DIV
	movfw	REG_Y
	call	Display_digit_char
	
	; send comma
	movlw	'.'
	call	Display_write_char
	
	; send the two decial digits behind the comma
	movlw	.0
	movwf	REG_Z+1
	movfw	Temp+2
	movwf	REG_Z

	movlw	HIGH 	.10
	movwf	REG_X+1
	movlw	LOW 	.10
	movwf	REG_X
	call	M_DIV
	movfw	REG_Y
	call	Display_digit_char

	movlw	HIGH 	.1
	movwf	REG_X+1
	movlw	LOW 	.1
	movwf	REG_X
	call	M_DIV
	movfw	REG_Y
	call	Display_digit_char
	
	return
	
Convert_to_c_degrees
	; multiply value in Temp by 2
	; load Temp into Y
	movfw	Temp+1
	movwf	REG_Y+1
	movfw	Temp
	movwf	REG_Y
	; load 2 into X
	movlw	HIGH 	.2
	movwf	REG_X+1
	movlw	LOW 	.2
	movwf	REG_X
	
	call	M_MUL	; X * Y => Z

	bcf		Temp+3, 0	; we assume that result will be positive for now

	; substract 27315 from the result
	; load 27315 into X
	movlw	HIGH	.27315
	movwf	REG_X+1
	movlw	LOW		.27315
	movwf	REG_X
	
	call	M_SUB	; Z - X => Z

	; did register onderflow?
	btfss	REG_Z+1, 7
	goto	Convert_to_c_degrees_cnt
	; yes it did, convert to posibive value again
	COMF	REG_Z, F	; inverse all bits
	COMF	REG_Z+1, F  ; inverse all bits
	movlw	.0			; and add 1
	movwf	REG_X+1		;  "
	movlw	.1			;  "
	movwf	REG_X		;  "
	call	M_ADD       ;  "
	bsf		Temp+3, 0	; mark value as negative after all

Convert_to_c_degrees_cnt
	
	; devide by 100
	movlw	HIGH	.100
	movwf	REG_X+1
	movlw	LOW		.100
	movwf	REG_X
	
	call	M_DIV
	
	; put the result back into Temp; Temp+2 contains the value 
	; behind the command and Temp+3,0 falgs whether it is pos or neg
	movfw	REG_Y+1
	movwf	Temp+1
	movfw	REG_Y
	movwf	Temp
	movfw	REG_Z
	movwf	Temp+2

	return
	

Delay_1s
	if CLOCKSPEED == .8000000
		;1999996 cycles
		movlw	0x11
		movwf	DelayCounter1
		movlw	0x5D
		movwf	DelayCounter2
		movlw	0x05
		movwf	DelayCounter3
	else
		if CLOCKSPEED == .4000000
				;999990 cycles
			movlw	0x07
			movwf	DelayCounter1
			movlw	0x2F
			movwf	DelayCounter2
			movlw	0x03
			movwf	DelayCounter3
		else
			error "Unsupported clockspeed"
		endif
	endif
Delay_1s_loop
	decfsz	DelayCounter1, f
	goto	$+2
	decfsz	DelayCounter2, f
	goto	$+2
	decfsz	DelayCounter3, f
	goto	Delay_1s_loop

			;4 cycles (including call)
	return
		
	end